@@ -58,6 +58,8 @@ public:
   virtual std::set<simdutf::encoding_type>
   expected_encodings(const std::string &procedure) override;
 
+  void list_procedures(ListingMode) const;
+
 protected:
   virtual void run(const std::string &procedure_name,
                    size_t iterations) override;
@@ -100,6 +102,8 @@ private:
                                  size_t iterations);
   void run_validate_utf32(const simdutf::implementation &implementation,
                           size_t iterations);
+  void run_to_well_formed_utf16(const simdutf::implementation &implementation,
+                                size_t iterations);
   void
   run_validate_utf32_with_errors(const simdutf::implementation &implementation,
                                  size_t iterations);
@@ -110,8 +114,16 @@ private:
   void
   run_utf8_length_from_latin1(const simdutf::implementation &implementation,
                               size_t iterations);
+  void
+  run_utf8_length_from_utf16le(const simdutf::implementation &implementation,
+                               size_t iterations);
+  void
+  run_utf8_length_from_utf16be(const simdutf::implementation &implementation,
+                               size_t iterations);
   void run_utf8_length_from_utf32(const simdutf::implementation &implementation,
                                   size_t iterations);
+  void run_utf16_length_from_utf8(const simdutf::implementation &implementation,
+                                  size_t iterations);
   void run_convert_latin1_to_utf8(const simdutf::implementation &implementation,
                                   size_t iterations);
   void
@@ -180,12 +192,18 @@ private:
   void
   run_convert_valid_utf32_to_utf8(const simdutf::implementation &implementation,
                                   size_t iterations);
+
+  template <endianness byte_order>
   void run_convert_utf32_to_utf16(const simdutf::implementation &implementation,
                                   size_t iterations);
+  template <endianness byte_order>
   void run_convert_utf32_to_utf16_with_errors(
       const simdutf::implementation &implementation, size_t iterations);
+
+  template <endianness byte_order>
   void run_convert_valid_utf32_to_utf16(
       const simdutf::implementation &implementation, size_t iterations);
+
   void run_convert_valid_utf16_to_utf32(
       const simdutf::implementation &implementation, size_t iterations);
   void run_detect_encodings(const simdutf::implementation &implementation,
